% Example of script for for creating "onsets" arrays (for SPM First_Level analysis) from a LabView log file
% Author: Samuel Planton and Valrie Chanoine(Brain and Language Institute (http://www.blri.fr/))
% Date: October 10, 2016

close all; clear;clc;
%%
% ------- fMRI Timings --------- %
nb_slices_TR = 54/3; % 54 slices multiband 3
TR = 1.224;
slice_dur = TR/nb_slices_TR;
% ------------------------------ %

%%
%============================================================%
% --------------- Extract Data from Log File --------------- %
%============================================================%

[filename, fullDir] = uigetfile('*.txt','Log File');

delimiter = '\t';
formatSpec = '%s%s%s%s%s%s%s%s%s%s%s%s%s%s%[^\n\r]';
fileID = fopen([fullDir filename],'r');
dataArray = textscan(fileID, formatSpec, 'Delimiter', delimiter,  'ReturnOnError', false);

% Convert the contents of columns containing numeric strings to numbers.
% Replace non-numeric strings with NaN.
raw = repmat({''},length(dataArray{1}),length(dataArray)-1);
for col=1:length(dataArray)-1
    raw(1:length(dataArray{col}),col) = dataArray{col};
end
numericData = NaN(size(dataArray{1},1),size(dataArray,2));
for col=[3,5,6,7,8,9,10,11,12,13,14]
    % Converts strings in the input cell array to numbers. Replaced non-numeric
    % strings with NaN.
    rawData = dataArray{col};
    for row=1:size(rawData, 1);
        % Create a regular expression to detect and remove non-numeric prefixes and
        % suffixes.
        regexstr = '(?<prefix>.*?)(?<numbers>([-]*(\d+[\,]*)+[\.]{0,1}\d*[eEdD]{0,1}[-+]*\d*[i]{0,1})|([-]*(\d+[\,]*)*[\.]{1,1}\d+[eEdD]{0,1}[-+]*\d*[i]{0,1}))(?<suffix>.*)';
        try
            result = regexp(rawData{row}, regexstr, 'names');
            numbers = result.numbers;
            
            % Detected commas in non-thousand locations.
            invalidThousandsSeparator = false;
            if any(numbers==',');
                thousandsRegExp = '^\d+?(\,\d{3})*\.{0,1}\d*$';
                if isempty(regexp(thousandsRegExp, ',', 'once'));
                    numbers = NaN;
                    invalidThousandsSeparator = true;
                end
            end
            % Convert numeric strings to numbers.
            if ~invalidThousandsSeparator;
                numbers = textscan(strrep(numbers, ',', ''), '%f');
                numericData(row, col) = numbers{1};
                raw{row, col} = numbers{1};
            end
        catch me
        end
    end
end

% Split data into numeric and cell columns.
rawNumericColumns = raw(:, [3,5,6,7,8,9,10,11,12,13,14]);
rawCellColumns = raw(:, [1,2,4]);

% Replace non-numeric cells with NaN
R = cellfun(@(x) ~isnumeric(x) && ~islogical(x),rawNumericColumns); % Find non-numeric cells
rawNumericColumns(R) = {NaN}; % Replace non-numeric cells

% Allocate imported array to column variable names
CONDITION = rawCellColumns(:, 1);
% TEXT = rawCellColumns(:, 2);
% BMP_NAME = cell2mat(rawNumericColumns(:, 1));
% RIEN = rawCellColumns(:, 3);
DURATION_SLICES = cell2mat(rawNumericColumns(:, 2));
% RESPONSE = cell2mat(rawNumericColumns(:, 3));
% RIEN1 = cell2mat(rawNumericColumns(:, 4));
% GONOGO = cell2mat(rawNumericColumns(:, 5));
ONSETS_TRIGGERS = cell2mat(rawNumericColumns(:, 6));
ONSETS_MS = cell2mat(rawNumericColumns(:, 7));
% REPONSE_1 = cell2mat(rawNumericColumns(:, 8));
% JUSTE = cell2mat(rawNumericColumns(:, 9));
% INSTANT_REPONSE = cell2mat(rawNumericColumns(:, 10));
% TEMPS_REACTION = cell2mat(rawNumericColumns(:, 11));

% Clear temporary variables
clearvars delimiter formatSpec fileID dataArray ans raw col numericData rawData row regexstr result numbers invalidThousandsSeparator thousandsRegExp me  R;

%%
%============================================================%
% ----- Create onset*.mat SPM file for "event" analysis ---- %
%============================================================%

names = {'FIXATION','WORDS','NONWORDS'};
onsets      = {};
durations   = {};

for c=1:numel(names)
    I = find(strcmp(CONDITION,names{c}));
    onsets{c} = (ONSETS_MS(I))/1000; % in seconds
    durations{c} =  DURATION_SLICES(I)/nb_slices_TR;
end

%============================================================%
% Extract Data from "Tapping" Log File (i.e. manual responses)

filename_tapp = [filename(1: (length(filename)-4))  '_Tapping.txt'];
delimiter = '\t';
startRow = 2;
formatSpec = '%f%f%f%[^\n\r]';
fileID = fopen([fullDir filename_tapp],'r');
dataArray = textscan(fileID, formatSpec, 'Delimiter', delimiter, 'HeaderLines' ,startRow-1, 'ReturnOnError', false);
fclose(fileID);

CURRENT_TRIGGER1 = dataArray{:, 1};
ONSETS_MS1 = dataArray{:, 2};
DEVICE_CODE1 = dataArray{:, 3};

names{c+1} = 'RESPONSE';
onsets{c+1} = ONSETS_MS1/1000;
durations{c+1} = 0;

%============================================================%
% Save MAT file

fichier_out = fullfile(fullDir,[filename,'_onsets_event.mat']);
save (fichier_out, 'names', 'onsets', 'durations')

%%
%============================================================%
% ------ Create onset*.mat SPM file for block analysis ----- %
%============================================================%

names = {'FIXATION','WORDS','NONWORDS'};
onsets      = {};
durations   = {};

for c=1:numel(names)
    % For block onsets (1 trial = 2 lines)
    I = find(strcmp(CONDITION,names{c})); % Find all lines corresponding to the condition
    listON=[I(1)]; % I(1) = First appearance of the condition
    for i=2:numel(I) % Identify 1st trials of each block
        if I(i-1)-I(i)>-3; % Should not be more than two lines between two trials of the same block
            i=i+1;
        else
            listON = [listON I(i)];
        end;
    end;
    onsets{c} = ONSETS_TRIGGERS(listON)*slice_dur;

    % For block durations (1 block = 8 lines)          
    durations{c} = [];
    for i = 1 : numel (listON)
        durations{c} = [durations{c} sum(DURATION_SLICES ( listON(i):listON(i)+47 ))*slice_dur]; % Sum of 48 rows from the block onset (24 trials)
    end;           
end

%============================================================%
% Extract Data from "Tapping" Log File (i.e. manual responses)

filename_tapp = [filename(1: (length(filename)-4))  '_Tapping.txt'];
delimiter = '\t';
startRow = 2;
formatSpec = '%f%f%f%[^\n\r]';
fileID = fopen([fullDir filename_tapp],'r');
dataArray = textscan(fileID, formatSpec, 'Delimiter', delimiter, 'HeaderLines' ,startRow-1, 'ReturnOnError', false);
fclose(fileID);

CURRENT_TRIGGER1 = dataArray{:, 1};
ONSETS_MS1 = dataArray{:, 2};
DEVICE_CODE1 = dataArray{:, 3};

names{c+1} = 'RESPONSE';
onsets{c+1} = ONSETS_MS1/1000;
durations{c+1} = 0;

%============================================================%
% Save MAT files
fichier_out = fullfile(fullDir,[filename,'_onsets_block.mat']);
save (fichier_out, 'names', 'onsets', 'durations')

%%
msgbox('======== Done ! ========')